@@ -537,12 +537,19 @@ pub fn try_build(attributes: Attributes) -> Result<()> {
         res.set("FileVersion", version_str);
         res.set("ProductVersion", version_str);
       }
+
       if let Some(product_name) = &config.package.product_name {
         res.set("ProductName", product_name);
       }
-      if let Some(short_description) = &config.tauri.bundle.short_description {
-        res.set("FileDescription", short_description);
-      }
+
+      let file_description = config
+        .package
+        .product_name
+        .or_else(|| manifest.package.as_ref().map(|p| p.name.clone()))
+        .or_else(|| std::env::var("CARGO_PKG_NAME").ok());
+
+      res.set("FileDescription", &file_description.unwrap());
+
       if let Some(copyright) = &config.tauri.bundle.copyright {
         res.set("LegalCopyright", copyright);
       }
